/*
**************** This version Modified 11/3/2017 by George Thomas, W5JDX for use with TFT LCD heads up display ***********
* Unnecessary code for this particular usage was removed, and some additional code was added.
* Thanks to James Buck, VE3BUX for the original FT857D Arduino library
* Additional code was also added for the ili9481 LCD display & from the Adafruit_GFX library & Andrew Mascolo's
* CountUpDown timer.
* ************************************************************************************************************************
  Created:  2012.08.16
   Author:  James Buck, VE3BUX
      Web:  http://www.ve3bux.com

  If you use this code and / or the associated library, all I ask is that you "give a nod" by making a small mention
  in your code's documentation section. I've worked hard on getting this working, so a little recognition is appreciated. :)
 
  Attach your radio's CAT interface to the following arduino pins to begin. Note I am using an Arduino Mega.:

    Radio CAT port GND -> Arduino GND
    Radio CAT port TX  -> Arduino pin 52
    Radio CAT port RX  -> Arduino pin 53
*/

#include <SoftwareSerial.h>
#include "FT857D.h"
#include"CountUpDownTimer.h"

CountUpDownTimer T(UP, HIGH); // Default precision is HIGH, but you can change it to also be LOW

FT857D radio;           // define "radio" so that we may pass CAT commands

//LCD definitions
#define RGB2LCD(r, g, b) (r << 16 | g << 8 | b)

#define LCD_RED    0xf800
#define LCD_GREEN  0x07E0
#define LCD_BLUE   0x001F
#define LCD_BLACK  0x0000
#define LCD_GRAY  0x9999
#define LCD_WHITE  0xffff
#define LCD_DarkBlue 0x1111
#define LCD_PURPLE 0xcc33
#define LCD_YELLOW 0xff00
#define LCD_ORANGE 0xcc00

#include "ili9481.h"
LcdIli9481 lcd;

//previously read data
long mode;
long lastmode;
String lastfreq;
bool transmitting;
bool lastRXTX;
int lastsMeter;
int lastwatts;
String spaces = "   ";

void setup() {

/*---- Working on a way to allow easier user input of I/O pins for software serial emulation
  ---- Right now, it is necessary to edit the FT857D.cpp file and to edit the line which states:
  ---- extern SoftwareSerial rigCat(3,2); // rx,tx
  ---- change the rx & tx pins to values which suit your application and save the FT857D.cpp file
*/
  radio.begin(9600);    // as with Serial.begin(9600); we wish to start the software serial port
                        // so that we may control a radio via CAT commands
  RestartDisplay();     //startup the display
}

void loop() {         
  int dly = 100;            // delay for x milliseconds between commands
  delay(dly);

//get radio mode
  mode = radio.getMode();
  if(mode != lastmode)
  {
    DisplayMode(mode);
    lastmode = mode;
  }
  
//get RX/TX mode
  bool RXTX = radio.chkTX();
  if(RXTX != lastRXTX)
  {
    DisplayRXTX(RXTX);
    lastRXTX = RXTX;
  }
  
//get frequency to string
  String freqStr = String(radio.getFreqMode(), DEC);
  if(freqStr == "166666665")
  {
    lcd.drawText(" NO DATA! ", 15, 110, LCD_WHITE, 4);
  }
  else
  {
    if(freqStr != lastfreq)
    {
      DisplayFreq(freqStr);
      lastfreq = freqStr;
    }
  }

//get watts or sMeter reading  
  if (RXTX == true){
    //get TX Power -- need to scale to get more accurate wattage
    int watts = radio.chkTXpwr();
      if(watts != lastwatts)
      {
        DisplaysMeter(watts);
        lastwatts = watts;
      }
  }
  else
  {
    //get S Meter
    int sMeter = radio.getSmeter();
    if(sMeter != lastsMeter)
    {
      DisplaysMeter(sMeter);
      lastsMeter = sMeter;
    }
  }

//transmitting so display timer
  if(transmitting == true){    
    DisplayTxTimer();
  }
  
}

//get display mode
void DisplayMode(long mode)
{
  String modeStr;
  switch(mode){
    case 0:
      modeStr = "LSB  ";
      break;
    case 1:
      modeStr = "USB  ";
      break;
    case 2:
      modeStr = "CW   ";
      break;
    case 3:
      modeStr = "CWR  ";
      break;
    case 4:
      modeStr = "AM   ";
      break;
    case 6:
      modeStr = "WFM  ";
      break;
    case 8:
      modeStr = "FM   ";
      break;
    case 10:
      modeStr = "DIG  ";
      break;
    case 12:
      modeStr = "PKT  ";
      RestartDisplay();   //allow a way to force redraw of entire screen.
      break;
    case 136:
      modeStr = "FMN  ";
      break;
  }
  char cMode[modeStr.length() + 1];
  modeStr.toCharArray(cMode,modeStr.length() + 1);
  lcd.drawText(cMode, 50, 55, LCD_BLUE, 3);  
}

void DisplayRXTX(bool RXTX)
{
  String trStr;
  uint16_t trColor;
  lcd.drawFilledRect(15, 245, 118, 40, LCD_BLACK);   //clear s or watt text display
  lastsMeter = 255;//force it to be rewritten next time
  lastwatts = 255;//force it to be rewritten next time
  if (RXTX == true){
    trStr = "TX  ";
    trColor = LCD_RED;
    transmitting = true;
    T.StartTimer();}
  else{
    trStr = "RX  ";
    trColor = LCD_GREEN;
    transmitting = false;
    T.StopTimer();
    lcd.drawText("     ", 100, 195, LCD_RED, 2);  //hide timer
    }
  char cTr[4];
  trStr.toCharArray(cTr,4);
  lcd.drawText(cTr, 370, 55, trColor, 3);
}

void DisplayTxTimer()
{
  T.Timer(); // run the timer
  
  if (T.TimeHasChanged() ) // this prevents the time from being constantly shown.
  {
    String min = (String)T.ShowMinutes();
    if(min.length() == 1){
      min = "0" + min;
    }    
    String sec = (String)T.ShowSeconds();
    if(sec.length() == 1){
      sec = "0" + sec;
    }
    String mmss = min + ":" + sec;
    char cTime[mmss.length() + 1];
    mmss.toCharArray(cTime,mmss.length() + 1);    
    lcd.drawText(cTime, 100, 195, LCD_RED, 2);  //display timer
  }
}

void DisplayFreq(String freqStr)
{
  String pad = "";
  String freqString = pad + freqStr.substring(0, freqStr.length() - 5) + "." + freqStr.substring(freqStr.length() - 5, freqStr.length() - 2) + "." + freqStr.substring(freqStr.length() - 2, freqStr.length());
  if(freqString.length() < 10){ //right justify string
    pad = spaces.substring(0, 10 - freqString.length());
    freqString = pad + freqString;
    }
  char cFreq[freqString.length() + 1];
  freqString.toCharArray(cFreq,freqString.length() + 1);
  lcd.drawText(cFreq, 15, 110, LCD_WHITE, 4);  
}

void DisplaysMeter(int meter)
{
  lcd.drawFilledRect(15, 270, 118, 40, LCD_BLACK);   //clear s or watt text display
  if (lastRXTX == true){  //TX
    if(mode == 4){meter = meter * 2;}
    DrawMeter(140, (meter * 3.1), LCD_RED);
    lastsMeter = 255; //force it to be rewritten next time
  }
  else  //RX
  {
    if(meter > 14 || mode == 6){ //blank out s meter for mode that don't display it
      lcd.drawFilledRect(140, 275, 310, 26, LCD_BLACK);   
    }
    else{
      String sStr = "s" + String(meter) + "  ";
      char cS[5];
      sStr.toCharArray(cS,5);
      lcd.drawText(cS, 20, 250, LCD_YELLOW, 3);  
      DrawMeter(140, (meter * 22), LCD_GREEN);
      lastwatts = 255; //force it to be rewritten next time
    }
  }
}

//draw the meter bar, padding out to full length with black
//maximum value is 310
void DrawMeter(uint16_t start, uint16_t value, uint16_t color)
{  
  lcd.drawFilledRect(start, 275, value, 26, color);   //draw meter bar
  
  if(value < 310)  //draw black padding bar
  {
    start = value + start;
    int width = 310 - value;
    lcd.drawFilledRect(start, 275, width, 26, LCD_BLACK);   
  }
}

void RestartDisplay()
{
  lcd.begin();
  lcd.rotate(90);
  lcd.clear(LCD_BLACK);
  lastmode = 255;
  lastRXTX = 255;
  lastfreq = 255;
  lastwatts = 255;
  lastsMeter = 255;
  lcd.drawText("George Thomas - W5JDX", 15, 12, LCD_ORANGE, 1);
  lcd.drawText("MHz", 370, 195, LCD_WHITE, 2);
  lcd.drawRect(137, 272, 315, 32, LCD_GRAY);
}

